from arclet.entari import BasicConfModel, plugin_config


class Config(BasicConfModel):
    to_me_only: bool = False
    """是否仅记录提及 Bot 自己的消息"""
    record_send: bool = False
    """是否记录发送的消息"""


config = plugin_config(Config)



from arclet.entari import metadata

from .config import Config
from . import listener as listener
from .model import MessageRecord as MessageRecord
from .utils import get_message_records as get_message_records

metadata(
    name="聊天记录",
    author=[{"name": "KomoriDev", "email": "mute231010@gmail.com"}],
    version="0.1.0",
    description="聊天记录持久化",
    readme="README.md",
    config=Config,
)

__all__ = ["MessageRecord", "get_message_records"]



from arclet.entari import plugin, Session, MessageCreatedEvent, SendResponse

from entari_plugin_database import AsyncSession  # entari: plugin

from .config import config
from .model import MessageRecord


@plugin.listen(MessageCreatedEvent)
async def message_receive(
    session: Session,
    event: MessageCreatedEvent,
    db_session: AsyncSession,
    is_notice_me: bool = False,
    is_reply_me: bool = False,
) -> None:
    if config.to_me_only and not (is_notice_me or is_reply_me):
        return

    record = MessageRecord(
        user_id=event.user.id,
        channel_id=event.channel.id,
        platform=event.account.platform,
        time=session.event.timestamp,
        type="message",
        message_id=event.message.id,
        message=event.message.dump(),
        plain_text=event.content.extract_plain_text(),
    )

    db_session.add(record)
    await db_session.commit()


async def message_send(
    session: Session,
    event: SendResponse,
    db_session: AsyncSession,
) -> None:
    if event.session is None or event.session.event.message is None:
        return

    record = MessageRecord(
        user_id=event.account.self_id,
        channel_id=event.channel,
        platform=event.account.platform,
        time=session.event.timestamp,
        type="message_sent",
        message_id=event.session.event.message.id,
        message=event.result[0].dump(),
        plain_text=event.message.extract_plain_text(),
    )

    db_session.add(record)
    await db_session.commit()


if config.record_send:
    plugin.listen(SendResponse)(message_send)



from datetime import datetime
from typing import Literal

from sqlalchemy import String, JSON, TEXT

from arclet.entari import MessageObject, MessageChain, Element
from entari_plugin_database import Base, Mapped, mapped_column  # entari: plugin


class MessageRecord(Base):
    __tablename__ = "entari_plugin_chronicle_messagerecord"
    __table_args__ = {"extend_existing": True}

    id: Mapped[int] = mapped_column(primary_key=True)
    user_id: Mapped[str] = mapped_column(String(64))
    """用户id"""
    channel_id: Mapped[str] = mapped_column(String(64), nullable=True)
    """频道id"""
    platform: Mapped[str] = mapped_column(String(32))
    """平台"""

    message_id: Mapped[str] = mapped_column(String(255))
    """消息id"""
    message: Mapped[dict] = mapped_column(JSON)
    """消息内容"""
    plain_text: Mapped[str] = mapped_column(TEXT)
    """消息内容的纯文本形式"""

    type: Mapped[Literal["message", "message_sent"]] = mapped_column(String(32))
    """消息类型"""
    time: Mapped[datetime]
    """消息时间"""

    def get_message(self) -> MessageChain[Element]:
        msg_obj = MessageObject.parse(self.message)
        return MessageChain(msg_obj.message)



from datetime import datetime
from typing import Literal
from collections.abc import Iterable, Sequence

from entari_plugin_database import get_session
from sqlalchemy import select

from .model import MessageRecord


async def get_message_records(
    user_ids: list[str] | None = None,
    channel_ids: list[str] | None = None,
    platforms: Iterable[str] | None = None,
    time_start: datetime | None = None,
    time_end: datetime | None = None,
    types: Iterable[Literal["message", "message_sent"]] | None = None,
) -> Sequence[MessageRecord]:
    async with get_session() as db_session:
        query = select(MessageRecord)

        if user_ids:
            query = query.where(MessageRecord.user_id.in_(user_ids))

        if channel_ids:
            query = query.where(MessageRecord.channel_id.in_(channel_ids))

        if platforms:
            query = query.where(MessageRecord.platform.in_(list(platforms)))

        if time_start:
            query = query.where(MessageRecord.time >= time_start)

        if time_end:
            query = query.where(MessageRecord.time <= time_end)

        if types:
            query = query.where(MessageRecord.type.in_(list(types)))

        result = await db_session.execute(query)
        records = result.scalars().all()

        return records
